'use strict'

module.exports = function initialize(agent, events, moduleName, shim) {
  var tracer = shim.tracer
  var EventEmitter = events.EventEmitter
  var proto = EventEmitter.prototype

  if (shim.isWrapped(proto, 'emit')) {
    shim.logger.trace('EventEmitter already wrapped, not instrumenting.')
    return
  }

  // TODO: EventEmitter.init was added in Node 0.10.22. Once older versions are
  // deprecated, remove the `wrapClass` version of the instrumentation.
  if (EventEmitter.init) {
    shim.wrap(EventEmitter, 'init', function wrapInit(shim, init) {
      return function wrappedInit() {
        // All we want to do is make the stream context so the parent segment
        // will be the one that created the segment.
        shim.setInternalProperty(
          this,
          '__NR_streamContext',
          new StreamContext(shim.getSegment())
        )

        return init.apply(this, arguments)
      }
    })
  } else {
    shim.wrapClass(events, 'EventEmitter', function emitterPostConstructHook() {
      // All we want to do is make the stream context so the parent segment will
      // be the one that created the segment.
      shim.setInternalProperty(
        this,
        '__NR_streamContext',
        new StreamContext(shim.getSegment())
      )
    })
  }

  shim.wrap(proto, 'emit', function wrapEmit(shim, emit) {
    if (!shim.isFunction(emit)) {
      return emit
    }

    // NOTE: Below is a duplication of the logic in `shim.applySegment`. It is
    // done here because the alternative would be to iterate over the arguments
    // to copy them into a new array to hand to `shim.applySegment`.
    return function wrappedEmit(evnt, a, b, c, d) {
      var context = this.__NR_streamContext
      var eventInfo = context && context.getEvent(evnt)
      var segment = eventInfo && eventInfo.increment()
      if (!segment) {
        return emit.apply(this, arguments)
      }
      if (eventInfo.action) {
        eventInfo.action(segment, a, b, c, d)
      }

      var prevSegment = tracer.segment
      tracer.segment = segment
      try {
        return emit.apply(this, arguments)
      } finally {
        segment.touch()
        tracer.segment = prevSegment
      }
    }
  })

  function StreamContext(segment) {
    this.segment = segment
    this.eventSegments = Object.create(null)
  }

  StreamContext.prototype.getEvent = function getEvent(name) {
    if (!this.eventSegments.hasOwnProperty(name)) {
      this.eventSegments[name] = new EventInfo(this.segment, name)
    }
    return this.eventSegments[name]
  }

  StreamContext.prototype.on = function on(name, listener) {
    this.getEvent(name).action = listener
  }

  function EventInfo(parent, name) {
    this.parent = parent
    this.name = 'Event callback: ' + name
    this.count = 0
    this.segment = null
    this.action = null
  }

  EventInfo.prototype.increment = function increment() {
    return this.parent

    // TODO: Enable this generic event instrumentation.
    // if (!this.parent) {
    //   return null
    // }
    //
    // if (!this.segment) {
    //   this.segment = shim.createSegment(this.name, this.parent)
    //   this.segment.start()
    // }
    //
    // this.segment.setParameter('count', ++this.count)
    // return this.segment
  }
}
